!
! Copyright (c) 2014,2016 Yoshihide Yoshimoto
! 
program main
  implicit none
  integer,parameter:: nfin1 = 10, nfin2 = 11
  character(len=256):: ifname1, ifname2,argv(2)
  integer:: nkmesh1, nkmesh2, ntb1, ntb2, nocc1, nocc2
  real(kind=8),allocatable:: sklist1(:,:),sklist2(:,:)
  integer,allocatable:: n_band1(:),n_band2(:)
  complex(kind=8),allocatable:: amat1(:),amat2(:)

  real(kind=8),parameter:: eps = 1.0d-5
  integer:: ic, ib, jb, ik, ios
  logical:: ifdif
  real(kind=8):: delta

  if (iargc().eq.0) then
     write(6,*) 'eps = ',eps
     stop
  else if (iargc().ne.2) then
     write(6,*) 'error in args'
     stop 1
  end if

  ic = 1
  call getarg(ic, argv(ic))
  if (len_trim(argv(ic)) .eq. 0) then
     write(6,*) 'invalid fname1'
     stop 10
  end if
  ifname1 = argv(ic)
  ic = 2
  call getarg(ic, argv(ic))
  if (len_trim(argv(ic)) .eq. 0) then
     write(6,*) 'invalid fname2'
     stop 10
  end if
  ifname2 = argv(ic)

  open(nfin1, file=ifname1, status='old', action='read', form='unformatted', iostat=ios)
  if (ios.ne.0) then
     write(6,*) 'can not open fname1'
     stop 11
  end if
  open(nfin2, file=ifname2, status='old', action='read', form='unformatted', iostat=ios)
  if (ios.ne.0) then
     write(6,*) 'can not open fname2'
     stop 11
  end if
  rewind nfin1
  rewind nfin2

  read(nfin1) nkmesh1,ntb1,nocc1
  read(nfin2) nkmesh2,ntb2,nocc2
  if (nkmesh1.ne.nkmesh2) then
     ifdif = .true.
     write(6,*) 'number reducible k mesh'
     stop 1
  end if
  if (ntb1.ne.ntb2) then
     ifdif = .true.
     write(6,*) 'number total band'
     stop 1
  end if
  if (nocc1.ne.nocc2) then
     ifdif = .true.
     write(6,*) 'number occupied band'
     stop 1
  end if

  allocate(sklist1(3,nkmesh1),sklist2(3,nkmesh2), &
       n_band1(nkmesh1),n_band2(nkmesh2), &
       amat1(nocc1),amat2(nocc2))

  read(nfin1) sklist1
  read(nfin2) sklist2

  do ik=1,nkmesh1
     if (maxval(abs(sklist2(:,ik)-sklist1(:,ik))).gt.eps) then
        write(6,*) 'sklist'
        stop 1
     end if
  end do

  read(nfin1) n_band1
  read(nfin2) n_band2

  do ik=1,nkmesh1
     if (n_band2(ik) .ne. n_band1(ik)) then
        write(6,*) 'n_band'
        stop 1
     end if
  end do

  do ik=1,nkmesh1
     do ib=1,ntb1
        read(nfin1) (amat1(jb),jb=1,nocc1)
        read(nfin2) (amat2(jb),jb=1,nocc2)
     end do
     delta = maxval(abs(dble(amat1(:)-amat2(:))) &
          + abs(aimag(amat1(:)-amat2(:))))
     if (delta.gt.eps) then
        write(6,*) 'amat',delta
        ifdif = .true.
        stop 1
     end if
  end do
  stop
end program main
