!
! Copyright (c) 2013 Yoshihide Yoshimoto
! 
program main
  implicit none
  integer,parameter:: nfin1 = 10, nfin2 = 11
  integer:: nbk1, nbk2, nw1, nw2
  character(len=256):: ifname1, ifname2,argv(2)
  integer,parameter:: nkptl = 6
  character(len=nkptl),allocatable:: nkpt1(:), nkpt2(:)
  integer,allocatable:: nkfi1(:), nkfi2(:)
  real(kind=8),allocatable:: ee1(:,:), ee2(:,:),sk1(:,:),sk2(:,:)
  integer:: ios, ic, nb, nk, nke, is, i
  integer:: nspin1, nspin2
  logical:: ifdif
  real(kind=8):: diffval

  real(kind=8),parameter:: epsee = 1.0d-5, epssk = 1.0d-5

  if (iargc().eq.0) then
     write(6,*) 'eps_ee = ',epsee
     write(6,*) 'eps_sk = ',epssk
     stop
  else if (iargc().ne.2) then
     write(6,*) 'error in args'
     stop 1
  end if

  ic = 1
  call getarg(ic, argv(ic))
  if (len_trim(argv(ic)) .eq. 0) then
     write(6,*) 'invalid fname1'
     stop 10
  end if
  ifname1 = argv(ic)
  ic = 2
  call getarg(ic, argv(ic))
  if (len_trim(argv(ic)) .eq. 0) then
     write(6,*) 'invalid fname2'
     stop 10
  end if
  ifname2 = argv(ic)

  open(nfin1, file=ifname1, status='old', action='read', form='formatted', iostat=ios)
  if (ios.ne.0) then
     write(6,*) 'can not open fname1'
     stop 11
  end if
  open(nfin2, file=ifname2, status='old', action='read', form='formatted', iostat=ios)
  if (ios.ne.0) then
     write(6,*) 'can not open fname2'
     stop 11
  end if
  rewind nfin1
  rewind nfin2

  read(nfin1,*) nbk1,nw1,nspin1
  read(nfin2,*) nbk2,nw2,nspin2
  if (nbk1.ne.nbk2) then
     ifdif = .true.
     write(6,*) 'number trace block'
     stop 1
  end if
  if (nw1.ne.nw2) then
     ifdif = .true.
     write(6,*) 'number band'
     stop 1
  end if
  if (nspin1.ne.nspin2) then
     ifdif = .true.
     write(6,*) 'number spin'
     stop 1
  end if
  allocate(nkpt1(nbk1+1),nkfi1(nbk1),nkpt2(nbk2+1),nkfi2(nbk2))
  allocate(sk1(3,nbk1+1),sk2(3,nbk2+1))

  read(nfin1,*) nkpt1
  read(nfin2,*) nkpt2
  do i=1,nbk1+1
     if (nkpt1(i).ne.nkpt2(i)) then
        ifdif = .true.
        write(6,*) 'name k point'
        stop 1
     end if
  end do

  read(nfin1,*) nkfi1
  read(nfin2,*) nkfi2
  do i=1,nbk1
     if (nkfi1(i).ne.nkfi2(i)) then
        ifdif = .true.
        write(6,*) 'number block division'
        stop 1
     end if
  end do

  do i=1,nbk1+1
     read(nfin1,*) sk1(1,i),sk1(2,i),sk1(3,i)
  end do
  do i=1,nbk2+1
     read(nfin2,*) sk2(1,i),sk2(2,i),sk2(3,i)
  end do
  if (maxval(abs(sk1-sk2)) .gt. epssk) then
     write(6,*) 'position traced k point'
     stop 1
  end if

  allocate(ee1(nw1,nspin1),ee2(nw2,nspin2))

  do nb=1,nbk1
     nke=nkfi1(nb)
     if (nb.eq.nbk1) nke = nke+1
     do nk=1,nke
        do is=1,nspin1
           read(nfin1,*) ee1(:,is)
        end do
        do is=1,nspin2
           read(nfin2,*) ee2(:,is)
        end do
        diffval = maxval(abs(ee1-ee2))
        if (diffval .gt. epsee) then
           write(6,*) 'eigen values at k point: ', diffval
           stop 1
        end if
     end do
  end do
  stop
end program main
