!
! Copyright (c) 2013 Yoshihide Yoshimoto
! 
#include "fftgrid.h"

program main
  use fft_3d
  use util
  integer,parameter:: nfin = 10, nfout = 11
  integer:: nrx2, nry2, nrz2, nrx, nry, nrz, nex2, ney2, nez2
  real(kind=8):: aa(3,3),aax(3,3)
  integer:: ix(3,3)
  character(len=256):: ifname, ofname, name, argv(4)
  integer:: ios
  logical:: error
  integer:: iorg(3)
  real(kind=8),allocatable:: field(:,:,:,:),ofield(:,:,:,:)
  real(kind=8),allocatable:: rho(:,:,:,:),exrho(:,:,:,:)
  real(kind=8),allocatable:: tmp(:,:,:,:),extmp(:,:,:,:)
  character(len=8),parameter:: magic = 'TAPP:rho'
  character(len=8):: sign
  integer:: ic
  logical:: expandq
  integer:: nspin, is
  integer:: i1,i2,i3,j1,j2,j3,k1,k2,k3

  type (fft3_struct):: fs

  ic = 1
  call getarg(ic, argv(ic))
  if (len_trim(argv(ic)) .eq. 0) then
     write(6,*) 'invalid fname'
     stop
  end if
  ifname = argv(ic)
  open(nfin, file=ifname, status='old', action='read', form='unformatted')

  iorg = 0

  ic = ic + 1
  call getarg(ic, argv(ic))
  read(argv(ic),*,iostat=ios) ix(1,1), ix(2,1), ix(3,1)
  if (ios .eq. 0) then
     error = .false.
     ic = ic + 1
     call getarg(ic, argv(ic))
     read(argv(ic),*,iostat=ios)  ix(1,2), ix(2,2), ix(3,2)
     error = error .or. (ios .ne. 0)
     ic = ic + 1
     call getarg(ic, argv(ic))
     read(argv(ic),*,iostat=ios)  ix(1,3), ix(2,3), ix(3,3)
     error = error .or. (ios .ne. 0)
     if (error) then
        write(6,*) 'invalid input for expand cell'
        stop
     end if
     expandq = .true.
     write(6,'(1x,a)') 'expanding cell'
  else
     ix = 0
     ix(1,1) = 1
     ix(2,2) = 1
     ix(3,3) = 1
     expandq = .false.
     write(6,'(1x,a)') 'unit cell'
  end if

  rewind nfin
  read(nfin) sign
  if (sign .ne. magic) then
     write(6,*) 'bad magic: ', sign
     stop
  end if

  read(nfin) aa, nspin
  read(nfin) nrx2, nry2, nrz2
  nrx = nrx2/2
  nry = nry2/2
  nrz = nrz2/2
  allocate(field(nrx2,nry2,nrz2,nspin))
  read(nfin) field

  aax = matmul(aa,ix)

  nex2 = 0
  ney2 = 0
  nez2 = 0
  do i3=1,nrz2
     do i2=1,nry2
        do i1=1,nrx2
           k1 = mod(i1-1+nrx,nrx2)-nrx
           k2 = mod(i2-1+nry,nry2)-nry
           k3 = mod(i3-1+nrz,nrz2)-nrz
           j1 = ix(1,1)*k1+ix(2,1)*k2+ix(3,1)*k3
           j2 = ix(1,2)*k1+ix(2,2)*k2+ix(3,2)*k3
           j3 = ix(1,3)*k1+ix(2,3)*k2+ix(3,3)*k3
           nex2 = max(abs(j1),nex2)
           ney2 = max(abs(j2),ney2)
           nez2 = max(abs(j3),nez2)
        end do
     end do
  end do
  nex2 = algn235(nex2*2,1)
  ney2 = algn235(ney2*2,1)
  nez2 = algn235(nez2*2,1)

  write(6,'(1x,a,3(1x,i5))') 'new ne[xyz] =',nex2,ney2,nez2

  allocate(FFT_A3D_DEFSZ(rho,nrx2,nry2,nrz2))
  allocate(FFT_A3D_DEFSZ(exrho,nex2,ney2,nez2))

  allocate(ofield(nex2,ney2,nez2,nspin))

  do is=1,nspin
     FFT_A3D_RL(rho,:,:,:) = field(:,:,:,is)
     FFT_A3D_IM(rho,:,:,:) = 0.0d0

     call fft3_init(nrx2,nry2,nrz2,nrx2,nry2,nrz2,fs)
     allocate(FFT_A3D_DEFSZ(tmp,nrx2,nry2,nrz2))
     call fft3_fw(fs,rho,tmp)
     call fft3_done(fs)
     deallocate(tmp)

     exrho = 0.0d0
     do i3=1,nrz2
        do i2=1,nry2
           do i1=1,nrx2
              k1 = mod(i1-1+nrx,nrx2)-nrx
              k2 = mod(i2-1+nry,nry2)-nry
              k3 = mod(i3-1+nrz,nrz2)-nrz
              j1 = ix(1,1)*k1+ix(2,1)*k2+ix(3,1)*k3
              j2 = ix(1,2)*k1+ix(2,2)*k2+ix(3,2)*k3
              j3 = ix(1,3)*k1+ix(2,3)*k2+ix(3,3)*k3
              k1 = mod(j1+nex2,nex2)+1
              k2 = mod(j2+ney2,ney2)+1
              k3 = mod(j3+nez2,nez2)+1
              FFT_A3D_RL(exrho,k1,k2,k3) = FFT_A3D_RL(rho,i1,i2,i3)
              FFT_A3D_IM(exrho,k1,k2,k3) = FFT_A3D_IM(rho,i1,i2,i3)
           end do
        end do
     end do

     call fft3_init(nex2,ney2,nez2,nex2,ney2,nez2,fs)
     allocate(FFT_A3D_DEFSZ(extmp,nex2,ney2,nez2))
     call fft3_bw(fs,exrho,extmp)
     call fft3_done(fs)
     deallocate(extmp)

     ofield(:,:,:,is) = FFT_A3D_RL(exrho,:,:,:)
  end do

  ofname=trim(ifname)//'.dx'//achar(0)
  name='TAPP rho'//achar(0)
  call outfldpr(ofname, name, nex2, ney2, nez2, aax, iorg, nspin, &
       nex2, ney2, nez2, ofield, 0)
  deallocate(field)

  stop
end program main
