/*
  Copyright (c) 2012-2013 Yoshihide Yoshimoto
 */

#include <stdio.h>
#include <stdlib.h>

#define EXTFORT(symbol) symbol##_

static int
modulo(int i, int j) {
	int q;
	q = i/j;
	if (i < 0) { q = q - 1; }
	return (i - q*j);
}

static int
msbq() {
	union {
		unsigned int i;
		unsigned char c[4];
	} test;

	test.i = 1;
	if ( test.c[0] != 0) {
		return 0;
	} else {
		return 1;
	}
}

void
EXTFORT(outfldprsk)(char *ofname, char *name,
		 int *nx, int *ny, int *nz, double aa[3][3], int iorg[3],
		 int *nlx, int *nly, int *nlz, double *field,
		 double sk[3], double *ee, int *wflt) {

    const double oneau = 0.529177249;

    double delta[3][3], origin[3];
    int i, j, k, ic;
	FILE *fp;
	char *endian;

	float tmp;

	fp = fopen(ofname, "w+");
	if (fp == NULL) {
		fprintf(stderr, "invalid ofname %s\n", ofname);
		exit(1);
	}

    fprintf(fp, "object 1 class gridpositions counts %d %d %d\n",
			*nx+1, *ny+1, *nz+1);
    for(i=0; i<3; i++) {
		origin[i] = oneau*(aa[0][i]*iorg[0]/(*nx) + aa[1][i]*iorg[1]/(*ny)
						   + aa[2][i]*iorg[2]/(*nz));
	}
    fprintf(fp, "origin %16.8e %16.8e %16.8e\n",
			origin[0], origin[1], origin[2]);
	for(i=0; i<3; i++) {
		delta[0][i] = oneau*aa[0][i]/(*nx);
		delta[1][i] = oneau*aa[1][i]/(*ny);
		delta[2][i] = oneau*aa[2][i]/(*nz);
	}
	fprintf(fp, "delta %16.8e %16.8e %16.8e\n",
			delta[0][0], delta[0][1], delta[0][2]);
	fprintf(fp, "delta %16.8e %16.8e %16.8e\n",
			delta[1][0], delta[1][1], delta[1][2]);
	fprintf(fp, "delta %16.8e %16.8e %16.8e\n",
			delta[2][0], delta[2][1], delta[2][2]);
    fprintf(fp,"object 2 class gridconnections counts %d %d %d\n",
			*nx+1, *ny+1, *nz+1);

	if (*wflt) {
		fprintf(fp, "object 3 class array type float rank 0\n");
	} else {
		fprintf(fp, "object 3 class array type double rank 0\n");
	}
	if (msbq()) {
		endian = "msb";
	} else {
		endian = "lsb";
	}
    fprintf(fp, "items %d %s ieee data follows\n",
			(*nx+1)*(*ny+1)*(*nz+1), endian);
	for(k=iorg[2]; k<= *nx+iorg[2]; k++) {
		for(j=iorg[1]; j<= *ny+iorg[1]; j++) {
			for(i=iorg[0]; i <= *nz+iorg[0]; i++) {
				ic = (*nlx)*(*nly)*modulo(i,*nz) +
					(*nlx)*modulo(j,*ny) + modulo(k,*nx);
				if (*wflt) {
					tmp = field[ic];
					fwrite(&tmp, sizeof(float), 1, fp);
				} else {
					fwrite(&field[ic], sizeof(double), 1, fp);
				}
			}
		}
	}
    fprintf(fp, "\n");
    fprintf(fp, "attribute \"dep\" string \"positions\"\n");

    fprintf(fp, "object 4 class field\n");
    fprintf(fp, "component \"positions\" value 1\n");
    fprintf(fp, "component \"connections\" value 2\n");
    fprintf(fp, "component \"data\" value 3\n");

	fprintf(fp, "object 5 class array type double rank 1 shape 4\n");
    fprintf(fp, "items 1 text data follows\n");
	fprintf(fp, "%24.16e %24.16e %24.16e %24.16e\n",sk[0],sk[1],sk[2],*ee);

    fprintf(fp, "object \"%s\" class group\n", name);
    fprintf(fp, "member \"rho\" 4\n");
    fprintf(fp, "member \"sk ee\" 5\n");
    fprintf(fp, "end\n");
    fclose(fp);
	return;
}
