!
! Copyright (c) 2000-2013,2016 Yoshihide Yoshimoto
!
program wfchg2pdos
  implicit none
  integer,parameter:: nfin = 10, nfout = 11
  real(kind=8),allocatable:: sk(:,:),chr(:,:,:,:,:,:),ee(:,:,:)
  integer:: nks,nw,ifbunp,nspin,nbba1,nbba2,nsph
  integer:: nkmesh,ncomp
  real(kind=8),allocatable:: bzwk(:)
  integer:: ib,iks,is,ic,id,i,isph
  integer:: idens
  real(kind=8),allocatable:: rmin(:),rmax(:),rzero(:,:)
  integer,allocatable:: iaxis(:),ndiv(:)
  integer:: maxndiv
  character(len=256):: argv(6), ifname, ofname
  integer:: neng
  real(kind=8):: emin, emax, eng, epseng
  real(kind=8):: fac,tdos(2)
  real(kind=8),allocatable:: pdos(:,:,:,:)
  real(kind=8),parameter:: pi = 3.14159265358979323846d0
  real(kind=8):: ef
  logical:: iffilsta

  integer:: idum,ios

  call getarg(1, argv(1))
  ifname = argv(1)
  open(nfin, file=ifname, status='old', action='read', form='formatted')
  ofname = trim(ifname)//'.pdos'
  open(nfout, file=ofname, status='unknown', action='write', form='formatted')
  call getarg(2, argv(2))
  read(argv(2),*) neng
  call getarg(3, argv(3))
  read(argv(3),*) emin
  call getarg(4, argv(4))
  read(argv(4),*) emax
  call getarg(5, argv(5))
  read(argv(5),*) epseng
  
  call getarg(6, argv(6))
  read(argv(6),*,iostat=ios) ef
  if (ios .eq. 0) then
     iffilsta = .true.
  else
     iffilsta = .false.
  end if

  write(nfout,'(1x,a,e18.11)') '# epseng = ', epseng
  
  read(nfin,*) nks, nw, ifbunp, nspin, nbba1, nbba2, nkmesh, ncomp
  write(nfout,'(1x,a,i4)') '# ifbunp = ', ifbunp
  if (nspin.eq.1) then
     write(nfout,'(1x,a)') '# nspin = 1'
  else if (nspin.eq.2) then
     write(nfout,'(1x,a)') '# nspin = 2'
  else
     write(6,'(1x,a)') 'invalid value in nspin'
     stop
  end if
  read(nfin,*) nsph
  write(nfout,'(1x,a,i4)') '# nsph = ', nsph
  allocate(rmin(nsph),rmax(nsph),ndiv(nsph),rzero(3,nsph),iaxis(nsph))
  maxndiv = 0
  do isph=1,nsph
     if (ifbunp.eq.1) then
        read(nfin,*) iaxis(isph),rmin(isph),rmax(isph),ndiv(isph)
        write(nfout,'(1x,a,i4,2(1x,e14.7),1x,i4)') &
             '# iaxis,rmin,rmax,ndiv = ', &
             iaxis(isph),rmin(isph),rmax(isph),ndiv(isph)
     else if(ifbunp.eq.2) then
        read(nfin,*) (rzero(i,isph),i=1,3),rmin(isph),rmax(isph),ndiv(isph)
        write(nfout,'(1x,a,5(1x,e14.7),1x,i4)') &
             '# rzero(3),rmin,rmax,ndiv = ', &
             rzero(:,isph),rmin(isph),rmax(isph),ndiv(isph)
     end if
     maxndiv = max(maxndiv,ndiv(isph))
  end do
  if (iffilsta) then
     write(nfout,'(1x,a,e14.7)') '# filled state only. ef = ',ef
  end if

  allocate(sk(3,nks),bzwk(nks))
  allocate(ee(nw,nks,nspin),chr(0:maxndiv,ncomp**2,nsph,nw,nks,nspin))
  allocate(pdos(0:maxndiv,ncomp**2,nsph,nspin))

  do is=1,nspin
     do iks=1,nks
        read(nfin,*) idum, sk(:,iks), bzwk(iks)
        if (idum.ne.iks) then
           write(6,'(1x,a)') 'idum != iks'
           stop
        end if
        do isph=1,nsph
           read(nfin,*) idum
           if (idum.ne.isph) then
              write(6,'(1x,a)') 'idum != isph'
              stop
           end if
           do ib=nbba1,nbba2
              read(nfin,*) idum, ee(ib,iks,is)
              read(nfin,*) ((chr(id,idens,isph,ib,iks,is),id=0,ndiv(isph)),idens=1,ncomp**2)
           end do
        end do
     end do
  end do

  do i=0,neng
     eng = emin + (emax-emin)/neng*i
     tdos = 0.0d0
     pdos = 0.0d0
     do is=1,nspin
        do iks=1,nks
           do ib=nbba1,nbba2
              fac = bzwk(iks)*epseng &
                   /( pi*(epseng**2 + (eng - ee(ib,iks,is))**2) )
              if (iffilsta .and. ee(ib,iks,is).gt.ef) then
                 fac = 0.0d0
              end if
              tdos(is) = tdos(is) + fac
              do isph=1,nsph
                 do idens=1,ncomp**2
                    do id=0,ndiv(isph)
                       pdos(id,idens,isph,is) = pdos(id,idens,isph,is) &
                            + chr(id,idens,isph,ib,iks,is)*fac
                    end do
                 end do
              end do
           end do
        end do
     end do
     if (nspin.eq.1) then
        write(nfout,'(100(1x,e14.7))') eng, 2.0d0*tdos(1)/nkmesh/nspin, &
             ((2.0d0*pdos(id,idens,isph,1)/nkmesh,id=0,ndiv(isph)),isph=1,nsph)
     else
        write(nfout,'(100(1x,e14.7))') eng, tdos(1)/nkmesh, &
             ((pdos(id,1,isph,1)/nkmesh,id=0,ndiv(isph)),isph=1,nsph), &
             tdos(2)/nkmesh, &
             ((pdos(id,1,isph,2)/nkmesh,id=0,ndiv(isph)),isph=1,nsph)
     end if
  end do

end program wfchg2pdos
